// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
//
// Module: Psfgen
// Author: Richard Saxton 
//
// Description: Main class for producing a PSF image
//
// Modifications: See ChangeLog
//

#ifndef Psfgen_H
#define Psfgen_H

/* General classes */
#include <iostream>

/* SAS-specific classes */
#include "Tracer.h"
#include "errstr.h"
#include "param.h"

#include "CalServer.h"
#include "Dal.h"
#include "DssBuilder.h"
#include "DatafileAttributes.h"

#include "SscWcsConvertor.h"

/****************************************************
 *
 * class Psfgen
 *
 ***************************************************/

class Psfgen
{
public:

  /* Constructor */
  Psfgen();

  /* Destructor */
  ~Psfgen();

  /* Method to calculate the tel coords */
  CoordPair findTelCoords(string &coordtype, double x, double y);

  /* Method to write a PSF extension block */
  void writePsf(vector<Image<float> > psfmap,
                      vector<float> weight,
                      vector<float> energy, string &psffile,
                      int xsize, int ysize);

  /* Method to write a block containing a PSF at a given energy */
  void makePsfExtension(DataSet *dataSet, string &extname,
                              Image<float> psfimage, string &senergy,
                              int xsize, int ysize);

  /* Method to set the accuracy level */
  AccuracyLevel getAccuracyLevel(string &slevel);

  /* Method to get the weights from the command line */
  vector<float> getWeights(unsigned nenergy);

  /* Methods to add header keywords */
  void addWcs(Array *a, int xsize, int ysize);
  void addPrimaryKeys(Array *a);

private:


  /* Member fields */
  DatafileAttributes *_atts;
  CoordPair _celpos;

};

#endif

