// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
#include <iostream>
#include <string>
#include "Dal.h"
#include <iomanip>

#define DIFF(x,y) (x > y ? x - y : y - x)

using namespace std;

int main(int argc, char * argv[])
{
	if (argc < 3) {
		cout << "Usage: " << argv[0] << " PSF1 PSF2 [--modify] [--verbose] [--sizeonly]" << endl;
		return 1;
	}

	bool ver = false;
	bool sonly = false;
	bool modify = false;

	unsigned i = 0;
	if (argc >= 4) {
		for (i = 3; i < (unsigned) argc; i++)
			if (argv[i][0] == '-' && argv[i][1] == '-') {
				string str(argv[i]+2);
				if (!str.compare("verbose")) ver = true;
				if (!str.compare("sizeonly")) sonly = true;
				if (!str.compare("modify")) modify = true;
			}
	}

	Dal::AccessMode mode = Dal::Read;

	if (modify) mode = Dal::Modify;

	DataSet * psf1 = dataSetServer->open(argv[1], mode);
	if (!psf1) {
		cout << "Cannot open " << argv[1] << endl;
		return 2;
	}

	DataSet * psf2 = dataSetServer->open(argv[2], mode);
	if (!psf2) {
		dataSetServer->close(psf1);
		cout << "Cannot open " << argv[2] << endl;
		return 2;
	}

	Array * psf1_array = psf1->array(0);
	Array * psf2_array = psf2->array(0);

	unsigned nElements1 = psf1_array->elements();
	unsigned nElements2 = psf2_array->elements();

	longVec size1 = psf1_array->size();
	longVec size2 = psf2_array->size();

	if (nElements1 != nElements2) {
		dataSetServer->close(psf1);
		dataSetServer->close(psf2);
		cout << argv[1] << " and " << argv[2] << " have different size. (" << size1[0] << "x" << size1[1] << " vs " << size2[0] << "x" << size2[1]  << ")" << endl;
		return 3;
	}

	if (sonly) {
		cout << argv[1] << " and " << argv[2] << " have same size. (" << size1[0] << "x" << size1[1] << ")" << endl;
		return 0;
	}

	real32 * data1 = psf1_array->data()->real32Data();
	real32 * data2 = psf2_array->data()->real32Data();

	unsigned match = 0;
	for (i = 0; i < nElements1; ++i) {
//RDS		unsigned long * temp1 = reinterpret_cast<unsigned long *>(data1+i);
//RDS		unsigned long * temp2 = reinterpret_cast<unsigned long *>(data2+i);

               // Change to be if difference > 1% 
		//if (*temp1!=*temp2 && DIFF(temp1[0], temp2[0]) > 2) {
//RDS		if (*temp1!=*temp2 && DIFF(temp1[0], temp2[0]) > 
     //RDS                            max(temp1[0], temp2[0])/100.0)  {
		if (DIFF(data1[i],data2[i]) > max(data1[i],data2[i])/100.0) {
			if (ver) cout << "row: " << static_cast<signed>(i/size1[1])-static_cast<signed>(size1[0]/2) << "\tcol: " << static_cast<signed>(i%size1[1])-static_cast<signed>(size1[1]/2) << "\t--\t" << data1[i] << "\tvs\t" << data2[i] << "\tdiff: " << DIFF(data1[i], data2[i]) << endl;
			data1[i] = data2[i] = 1000;
			++match;			
		}
	}

	dataSetServer->close(psf1);
	dataSetServer->close(psf2);

	if (match == 0) {
		cout << argv[1] << " and " << argv[2] << " are same." << endl;
		return 0;
	} else {
		cout << argv[1] << " and " << argv[2] << " have " << match << " difference(s)." << endl;
		return 4;
	}
}
